@@ -1,18 +1,20 @@
 import os
+from logging import Logger
 from typing import Any
 
 import github
 import yaml
+from github.GithubException import UnknownObjectException
 from simple_logger.logger import get_logger
 
-LOGGER = get_logger(name="config")
-
 
 class Config:
     def __init__(
         self,
+        logger: Logger | None = None,
         repository: str | None = None,
     ) -> None:
+        self.logger = logger or get_logger(name="config")
         self.data_dir: str = os.environ.get("WEBHOOK_SERVER_DATA_DIR", "/home/podman/data")
         self.config_path: str = os.path.join(self.data_dir, "config.yaml")
         self.repository = repository
@@ -29,39 +31,37 @@ class Config:
 
     @property
     def root_data(self) -> dict[str, Any]:
-        LOGGER.debug(f"Loading config file {self.config_path}")
-
         try:
             with open(self.config_path) as fd:
                 return yaml.safe_load(fd)
         except Exception:
-            LOGGER.error("Config file is empty")
+            self.logger.error(f"Config file is empty: {self.config_path}")
             return {}
 
     @property
     def repository_data(self) -> dict[str, Any]:
-        LOGGER.debug(f"Loading repository level config for repository {self.repository}")
         return self.root_data["repositories"].get(self.repository, {})
 
     def repository_local_data(self, github_api: github.Github, repository_full_name: str) -> dict[str, Any]:
-        LOGGER.debug(f"Loading local config for repository {repository_full_name}")
-
         if self.repository and repository_full_name:
             from webhook_server.utils.helpers import get_github_repo_api
 
             try:
-                repo = get_github_repo_api(github_api=github_api, repository=repository_full_name)
-                _path = repo.get_contents(".github-webhook-server.yaml")
+                repo = get_github_repo_api(github_app_api=github_api, repository=repository_full_name)
+                try:
+                    _path = repo.get_contents(".github-webhook-server.yaml")
+                except UnknownObjectException:
+                    return {}
+
                 config_file = _path[0] if isinstance(_path, list) else _path
                 repo_config = yaml.safe_load(config_file.decoded_content)
-                LOGGER.debug(f"Repository {repository_full_name} config: {repo_config}")
                 return repo_config
 
             except Exception as ex:
-                LOGGER.debug(f"Repository {repository_full_name} config file not found or error. {ex}")
+                self.logger.error(f"Repository {repository_full_name} config file not found or error. {ex}")
                 return {}
 
-        LOGGER.debug("self.repository or self.repository_full_name is not defined")
+        self.logger.error("self.repository or self.repository_full_name is not defined")
         return {}
 
     def get_value(self, value: str, return_on_none: Any = None, extra_dict: dict[str, Any] | None = None) -> Any:
