@@ -1,6 +1,8 @@
 from __future__ import annotations
 
+import asyncio
 import datetime
+import os
 import shlex
 import subprocess
 from concurrent.futures import Future, as_completed
@@ -14,14 +16,44 @@ from github.Repository import Repository
 from simple_logger.logger import get_logger
 
 from webhook_server.libs.config import Config
+from webhook_server.libs.exceptions import NoApiTokenError
 
 
-def get_logger_with_params(name: str, repository_name: str = "") -> Logger:
+def get_logger_with_params(
+    name: str,
+    repository_name: str = "",
+    mask_sensitive: bool = True,
+) -> Logger:
+    mask_sensitive_patterns: list[str] = [
+        "container_repository_password",
+        "-p",
+        "password",
+        "token",
+        "apikey",
+        "secret",
+    ]
+
     _config = Config(repository=repository_name)
 
     log_level: str = _config.get_value(value="log-level", return_on_none="INFO")
     log_file: str = _config.get_value(value="log-file")
-    return get_logger(name=name, filename=log_file, level=log_level, file_max_bytes=1048576 * 50)  # 50MB
+
+    if log_file and not log_file.startswith("/"):
+        log_file_path = os.path.join(_config.data_dir, "logs")
+
+        if not os.path.isdir(log_file_path):
+            os.makedirs(log_file_path, exist_ok=True)
+
+        log_file = os.path.join(log_file_path, log_file)
+
+    return get_logger(
+        name=name,
+        filename=log_file,
+        level=log_level,
+        file_max_bytes=1024 * 1024 * 10,
+        mask_sensitive=mask_sensitive,
+        mask_sensitive_patterns=mask_sensitive_patterns,
+    )
 
 
 def extract_key_from_dict(key: Any, _dict: dict[Any, Any]) -> Any:
@@ -38,19 +70,17 @@ def extract_key_from_dict(key: Any, _dict: dict[Any, Any]) -> Any:
                         yield result
 
 
-def get_github_repo_api(github_api: github.Github, repository: int | str) -> Repository:
-    return github_api.get_repo(repository)
+def get_github_repo_api(github_app_api: github.Github, repository: int | str) -> Repository:
+    logger = get_logger_with_params(name="helpers")
+    logger.debug(f"Get GitHub API for repository {repository}")
+
+    return github_app_api.get_repo(repository)
 
 
-def run_command(
+async def run_command(
     command: str,
     log_prefix: str,
     verify_stderr: bool = False,
-    shell: bool = False,
-    timeout: int | None = None,
-    capture_output: bool = True,
-    check: bool = False,
-    pipe: bool = False,
     **kwargs: Any,
 ) -> tuple[bool, Any, Any]:
     """
@@ -60,44 +90,28 @@ def run_command(
         command (str): Command to run
         log_prefix (str): Prefix for log messages
         verify_stderr (bool, default False): Check command stderr
-        shell (bool, default False): run subprocess with shell toggle
-        timeout (int, optional): Command wait timeout
-        capture_output (bool, default True): Capture command output
-        check (bool, default False):  If check is True and the exit code was non-zero, it raises a
-            CalledProcessError
-        pipe (bool, default False): If pipe is True, text and capture_output would be set to False. stdout and
-            stderr, would be set to subprocess.PIPE and passed to subprocess.run call
-
 
     Returns:
         tuple: True, out if command succeeded, False, err otherwise.
     """
     logger = get_logger_with_params(name="helpers")
-    text = True
     out_decoded: str = ""
     err_decoded: str = ""
-    if pipe:
-        capture_output = False
-        text = False
-        kwargs["stdout"] = subprocess.PIPE
-        kwargs["stderr"] = subprocess.PIPE
+    kwargs["stdout"] = subprocess.PIPE
+    kwargs["stderr"] = subprocess.PIPE
+
     try:
         logger.debug(f"{log_prefix} Running '{command}' command")
-        sub_process = subprocess.run(
-            shlex.split(command),
-            capture_output=capture_output,
-            check=check,
-            shell=shell,
-            text=text,
-            timeout=timeout,
+        command_list = shlex.split(command)
+
+        sub_process = await asyncio.create_subprocess_exec(
+            *command_list,
             **kwargs,
         )
-        out_decoded = (
-            sub_process.stdout.decode(errors="ignore") if isinstance(sub_process.stdout, bytes) else sub_process.stdout
-        )
-        err_decoded = (
-            sub_process.stderr.decode(errors="ignore") if isinstance(sub_process.stderr, bytes) else sub_process.stderr
-        )
+
+        stdout, stderr = await sub_process.communicate()
+        out_decoded = stdout.decode(errors="ignore") if isinstance(stdout, bytes) else stdout
+        err_decoded = stderr.decode(errors="ignore") if isinstance(stderr, bytes) else stderr
 
         error_msg = (
             f"{log_prefix} Failed to run '{command}'. "
@@ -114,6 +128,7 @@ def run_command(
             return False, out_decoded, err_decoded
 
         return True, out_decoded, err_decoded
+
     except Exception as ex:
         logger.error(f"{log_prefix} Failed to run '{command}' command: {ex}")
         return False, out_decoded, err_decoded
@@ -129,9 +144,7 @@ def get_apis_and_tokes_from_config(config: Config) -> list[tuple[github.Github,
     return apis_and_tokens
 
 
-def get_api_with_highest_rate_limit(
-    config: Config, repository_name: str = ""
-) -> tuple[github.Github | None, str | None, str]:
+def get_api_with_highest_rate_limit(config: Config, repository_name: str = "") -> tuple[github.Github, str, str]:
     """
     Get API with the highest rate limit
 
@@ -151,7 +164,7 @@ def get_api_with_highest_rate_limit(
 
     remaining = 0
 
-    msg = "Get API and token"
+    msg = "Get API and tokens"
 
     if repository_name:
         msg += f" for repository {repository_name}"
@@ -161,15 +174,28 @@ def get_api_with_highest_rate_limit(
     apis_and_tokens = get_apis_and_tokes_from_config(config=config)
 
     for _api, _token in apis_and_tokens:
-        _api_user = _api.get_user().login
-        rate_limit = _api.get_rate_limit()
-        if rate_limit.core.remaining > remaining:
-            remaining = rate_limit.core.remaining
-            api, token = _api, _token
+        if _api.rate_limiting[-1] == 60:
+            logger.warning("API has rate limit set to 60 which indicates an invalid token, skipping")
+            continue
+
+        try:
+            _api_user = _api.get_user().login
+        except Exception as ex:
+            logger.warning(f"Failed to get API user for API {_api}, skipping. {ex}")
+            continue
+
+        _rate_limit = _api.get_rate_limit()
+
+        if _rate_limit.core.remaining > remaining:
+            remaining = _rate_limit.core.remaining
+            api, token, _api_user, rate_limit = _api, _token, _api_user, _rate_limit
 
     if rate_limit:
         log_rate_limit(rate_limit=rate_limit, api_user=_api_user)
 
+    if not _api_user or not api or not token:
+        raise NoApiTokenError("Failed to get API with highest rate limit")
+
     logger.info(f"API user {_api_user} selected with highest rate limit: {remaining}")
     return api, token, _api_user
 
